/*	go.c		Paul Albrecht		Sept 1984
 
	Copyright (C) Paul Albrecht 1988.  All rights reserved.

	Last Update:	Nov 20, 1989
	EMACS_MODES:	tabstop=4

	Graphics primitives for Graphon and other Tektronix emulators.
*/

#include	"plt.h"
#include	<signal.h>
#include	<sgtty.h>


void	xmt();

static	double	xmin, ymin, xscl = 1, yscl = 1;
static	int		oldx, oldy;
static	char	ohiy = -1, ohix = -1, oloy = -1;

static	struct	{
	char	*pterm;
	PTERM	p;
	char	*open;
	char	*close;
	char	*erase;
	}	tty_codes[] = {
	{ "go140", 0, "\0331\033*\035", "\035\0332", "\033\014" },
	{ "go235", 0, "\0331\033*\035", "\035\0332", "\033\014" },
	{ "seltek",0, "\0332\033H\033J\0331\033*\035", "\035\0332", "\033\014" },
	{ "4014",  0, "\035",           "\035\0332", "\033\014" },
	}, *codes;


static	quit()
{
static	short	reset;

	signal( SIGINT, quit );

	if( !reset ) {
		reset = 1;
		closepl();
		exit( 1 );
	}
}


void	openpl()
{
	printf( codes->open );

	signal( SIGINT, quit );
}


void	closepl()
{
	if( codes != 0 ) {
		printf( codes->close );
		fflush( stdout );
	}
}


void	erase()
{
	printf( codes->erase );
	ohix = ohiy = oloy =  -1;
}


void	space( x0, y0, x1, y1 )
int		x0, y0, x1, y1;
{
	xmin = x0;
	ymin = y0;

	xscl = ((double)(codes->p)->xsquare-1)/(x1-x0);
	yscl = ((double)(codes->p)->yfull-1)/(y1-y0);
}


void	point( x, y )
int		x, y;
{
	move( x, y );
	cont( x, y );
}



void	line( x0, y0, x1, y1 )
int		x0, y0, x1, y1;
{
	move( x0, y0 );
	cont( x1, y1 );
}


void	label( s )
char	*s;
{
	putchar( 037 );
	while( *s ) {
		putchar( *s );
		s++;
	}
	putchar( 035 );
}


void	move( x, y )
int		x, y;
{
	putchar( 035 );
	cont( x, y );
}


void	cont( x, y )
int		x, y;
{
	oldx = (x-xmin) * xscl + 0.5;
	oldy = (y-ymin) * yscl + 0.5;
	xmt( oldx, oldy );
}


void	xmt( x, y )
int		x, y;
{
char	hix, hiy, lox, loy;

	hix = (x>>7) & 037;
	hiy = (y>>7) & 037;
	lox = (x>>2) & 037;
	loy = (y>>2) & 037;
	
	if( hiy != ohiy ) {
		putchar( hiy|040 );
		ohiy = hiy;
	}
	if( hix != ohix ){
		putchar( loy|0140 );
		putchar( hix|040 );
		ohix = hix;
		oloy = loy;
	}
	if( loy != oloy ) {
		putchar( loy|0140 );
		oloy = loy;
	}
	putchar( lox|0100 );
}

/************************************************************************/

void	special( what, arg0, arg1 )
Const		what;
PtrUnion	arg0, arg1;
{
	switch( what ) {
		case SETPTERM:
			PTERMLookup( pterm, NULLS );
			for( codes=tty_codes;  codes < ENDP(tty_codes);  codes++ ) {
				if( strcmp(pterm,codes->pterm) == 0 )
					break;
			}
			if( codes == ENDP(tty_codes) )
				codes = tty_codes;
			codes->p = p;
			break;
		case SETLINEMODE:
			linemod( arg0.c );
			break;
	}
}


void	linemod( pattern )
char	*pattern;
{
		if( strcmp(pattern,"dotted") == 0 )
			printf( "\033a" );
	else	if( strcmp(pattern,"dotdashed") == 0 )
			printf( "\033b" );
	else	if( strcmp(pattern,"shortdashed") == 0 )
			printf( "\033c" );
	else	if( strcmp(pattern,"longdashed") == 0 )
			printf( "\033d" );
	else	if( strcmp(pattern,"solid") == 0 )
			printf( "\033`" );
}
